@@ -1,8 +1,9 @@
 # App::hopen: Implementation of the hopen(1) program
 package App::hopen;
-our $VERSION = '0.000009'; # TRIAL
+our $VERSION = '0.000011';
 
 # Imports {{{1
+use strict;
 use Data::Hopen::Base;
 
 use App::hopen::AppUtil ':all';
@@ -20,6 +21,8 @@ use Hash::Merge;
 use Path::Class;
 use Scalar::Util qw(looks_like_number);
 
+BEGIN { $Data::Dumper::Indent = 1; }    # DEBUG
+
 # }}}1
 # Constants {{{1
 
@@ -41,7 +44,10 @@ use constant EXIT_PARAM_ERR => 2;   # couldn't understand the command line
 
 App::hopen - hopen build system command-line interface
 
-=head1 SYNOPSIS
+=head1 INTRODUCTION
+
+(Note: most features are not yet implemented ;) .  However it will generate
+a Makefile for a basic C<Hello, World> program at this point!)
 
 hopen is a cross-platform software build generator.  It makes files you can
 pass to Make, Ninja, Visual Studio, or other build tools, to compile and
@@ -70,7 +76,32 @@ See L<App::hopen::Conventions> for details of the input format.
 Why Perl?  Because (1) you probably already have it installed, and
 (2) it is the original write-once, run-everywhere language!
 
-=head1 USAGE
+=head2 Example
+
+Create a file C<.hopen.pl> in your source tree.  Then:
+
+    $ hopen
+    From ``.'' into ``built''
+    Running Check phase
+
+Now C<built/MY.hopen.pl> has been created, and loaded with information about
+your configuration.  You can edit that file if you want to change what will
+happen next.
+
+    $ hopen
+    From ``.'' into ``built''
+    Running Gen phase
+
+Now C<built/Makefile> has been created.
+
+    $ hopen --build
+    Building in foo/built
+
+And your software is ready to go!
+
+See L<App::hopen::Conventions> for information on writing C<.hopen.pl> files.
+
+=head1 SYNOPSIS
 
     hopen [options] [--] [destination dir [project dir]]
 
@@ -78,8 +109,80 @@ If no project directory is specified, the current directory is used.
 
 If no destination directory is specified, C<< <project dir>/built >> is used.
 
+See L<App::hopen> and L<App::hopen::Conventions> for more details.
+
+=head1 OPTIONS
+
+=over
+
+=item -a C<architecture>
+
+Specify the architecture.  This is an arbitrary string interpreted by the
+generator or toolset.
+
+=item -e C<Perl code>
+
+Add the C<Perl code> as if it were a hopen file.  C<-e> files are processed
+after all other hopen files, so can modify anything that has been set up
+by those files.  Can be specified more than once.
+
+=item --fresh
+
+Start a fresh build --- ignore any C<MY.hopen.pl> file that may exist in
+the destination directory.
+
+=item --from C<project dir>
+
+Specify the project directory.  Overrides a project directory given as a
+positional argument.
+
+=item -g C<generator>
+
+Specify the generator.  The given C<generator> should be either a full package
+name or the part after C<App::hopen::Gen::>.
+
+=item -t C<toolset>
+
+Specify the toolset.  The given C<toolset> should be either a full package
+name or the part after C<App::hopen::T::>.
+
+=item --to C<destination dir>
+
+Specify the destination directory.  Overrides a destination directory given
+as a positional argument.
+
+=item --phase C<phase>
+
+Specify which phase of the process to run.  Note that this overrides whatever
+is specified in any MY.hopen.pl file, so may cause unexpected results!
+
+If C<--phase> is given, no other hopen file can set the phase, and hopen will
+terminate if a file attempts to do so.
+
+=item -q
+
+Produce no output (quiet).  Overrides C<-v>.
+
+=item -v, --verbose=n
+
+Verbose.  Specify more C<v>'s for more verbosity.  At present, C<-vv>
+(equivalently, C<--verbose=2>) gives
+you detailed traces of the data, and C<-vvv> gives you more detailed
+code tracebacks on error.
+
+=item --version
+
+Print the version of hopen and exit
+
+=back
+
 =head1 INTERNALS
 
+After the C<hopen> file is processed, cycles are detected and reported as
+errors.  *(TODO change this to support LaTeX multi-run files?)*  Then the DAG
+is traversed, and each operation writes the necessary information to the
+file being generated.
+
 =cut
 
 # }}}1
@@ -100,7 +203,7 @@ my %CMDLINE_OPTS = (
     ARCHITECTURE => ['a','|A|architecture|platform=s'],
         # -A and --platform are for the comfort of folks migrating from CMake
 
-    #BUILD => ['build'],    # TODO implement this --- if specified, do not
+    BUILD => ['build'],     # If specified, do not
                             # run any phases.  Instead, run the
                             # build tool indicated by the generator.
 
@@ -115,6 +218,9 @@ my %CMDLINE_OPTS = (
 
     GENERATOR => ['g', '|G|generator=s', 'Make'],     # -G is from CMake
         # *** This is where the default generator is set ***
+        # TODO? add an option to pass parameters to the generator?
+        # E.g., which make(1) to use?  Or maybe that should be part of the
+        # ARCHITECTURE string.
 
     #GO => ['go'],  # TODO implement this --- if specified, run all phases
                     # and invoke the build tool without requiring the user to
@@ -205,7 +311,7 @@ values from the command line, keyed by the keys in L</%CMDLINE_OPTS>.
 
         Pod::Usage::pod2usage(
             -exitval => EXIT_OK, @in,
-            -verbose => 99, -sections => '!INTERNALS'   # suppress
+            -verbose => 99, -sections => '!INTERNALS'   # suppress INTERNALS
         ) if have('man');
     }
 
@@ -248,6 +354,9 @@ Set to truthy if MY.hopen.pl sets the phase.
 =cut
 
 our $_did_set_phase = false;
+    # Whether the current hopen file called set_phase()
+
+my $_hf_pkg_idx = 0;    # unique ID for the packages of hopen files
 
 sub _execute_hopen_file {       # Load and run a single hopen file {{{2
 
@@ -349,14 +458,14 @@ turned into a C<use lib> statement (see L<lib>) in the generated source.
         hlog { 'Processing', $fn->{name} };
         $file_text = $fn->{text};
         $friendly_name = $fn->{name};
-        $pkg_name = 'CmdLineE' . $fn->{num};
+        $pkg_name = 'CmdLineE' . $fn->{num} . '_' . $_hf_pkg_idx++;
         $phase_text .= defined($opts{phase}) ? $cannot_set_phase : $set_phase;
             # -e's can set phase unless --phase was specified
 
     } else {
         hlog { 'Processing', $fn };
         $file_text = file($fn)->slurp;
-        $pkg_name = ($fn =~ s/[^a-zA-Z0-9]/_/gr);
+        $pkg_name = ($fn =~ s/[^a-zA-Z0-9]/_/gr) . '_' . $_hf_pkg_idx++;
         $friendly_name = $fn;
 
         if( isMYH($fn) and !defined($opts{phase}) ) {
@@ -406,15 +515,20 @@ EOT
 EOT
     }
 
+    # Run the code given in the hopen file.  Wrap it in a named BLOCK so that
+    # Phases::on() will work, but don't rely on the return value of that
+    # BLOCK (per perlsyn).
+
     $src .= <<EOT;
 
     sub __Rsub_$pkg_name {
-        my \$__R_retval = do {   # return statements in here will exit the Rsub
-            __R_DO: {
+        my \$__R_retval;
+        __R_DO: {
+            \$__R_retval = do {   # return statements in here will exit the Rsub
 #line 1 "$friendly_name"
 $file_text
-            } #__R_DO
-        }; # do{}
+            }; # do{}
+        } #__R_DO
 EOT
 
     # If the file_text did not expressly return(), control will reach the
@@ -423,15 +537,31 @@ EOT
     # via Phases::on(), we have a defined __R_on_result.  If either of those
     # is defined, make sure it's not a DAG or GraphBuilder.  Those should not
     # be put into the return data.
+    #
+    # Also, any defined, non-hash value is ignored, so that we don't
+    # wind up with lots of hashes like { 1 => 1 }.
+    #
+    # If the file_text did expressly return(), whatever it returned will
+    # be used as-is.  Like perlref says, we are not totalitarians.
 
     $src .= <<EOT;
         \$__R_retval //= \$__R_on_result;
 
+        ## hlog { '__Rpkg_$pkg_name retval before checks',
+        ##        ref \$__R_retval, Dumper \$__R_retval} 3;
+
         if(defined(\$__R_retval) && ref(\$__R_retval)) {
             die 'Hopen files may not return graphs'
                 if eval { \$__R_retval->DOES('Data::Hopen::G::DAG') };
             die 'Hopen files may not return graph builders (is a ->goal or ->default_goal missing?)'
                 if eval { \$__R_retval->DOES('Data::Hopen::G::GraphBuilder') };
+
+        }
+
+        if(defined(\$__R_retval) and ref \$__R_retval ne 'HASH') {
+            warn ('Hopen files must return hashrefs; ignoring ' .
+                    lc(ref(\$__R_retval) || 'scalar')) unless \$QUIET;
+            \$__R_retval = undef;
         }
 
         return \$__R_retval;
@@ -451,7 +581,7 @@ EOT
     die "Error in $friendly_name: $@" if $@;
 
     # Get the data from the package we just ran
-    my $hrAddlData = eval ("\$__Rpkg_${pkg_name}" . '::hrNewData');
+    my $hrAddlData = eval ("\$__Rpkg_$pkg_name" . '::hrNewData');
 
     hlog { 'old data', Dumper($_hrData) } 3;
     hlog { 'new data', Dumper($hrAddlData) } 2;
@@ -511,7 +641,8 @@ be run if it is empty.
     hlog { 'Graph is', ($Build->empty ? 'empty.' : 'not empty.'),
             ' Final data is', Dumper($_hrData) } 2;
 
-    hlog { Data::Dumper->new([$Build], ['$Build'])->Indent(1)->Dump } 5;
+    hlog { 'Build graph', '' . $Build->_graph } 5;
+    hlog { Data::Dumper->new([$Build], ['$Build'])->Indent(1)->Dump } 9;
 
     # If there is no build graph, just return the data.  This is useful
     # enough for debugging that I am making it documented behaviour.
@@ -524,10 +655,12 @@ be run if it is empty.
     my $env = Data::Hopen::Scope::Environment->new(name => 'outermost');
     my $scope = Data::Hopen::Scope::Hash->new(name => 'from hopen files');
     $scope->adopt_hash($_hrData);
+    $scope->outer($env);    # make the environment accessible...
+    $scope->local(true);    # ... but not copied by local-scope calls.
 
     # Run the DAG
     my $result_data = $Build->run(-context => $scope, -phase => $Phase,
-                                    -generator => $Generator);
+                                    -visitor => $Generator);
     hlog { Data::Dumper->new([$result_data], ['Build graph result data'])->Indent(1)->Dump } 2;
     return $result_data;
 } #_run_phase() }}}2
@@ -539,21 +672,34 @@ sub _inner {    # Run a single invocation of hopen(1). {{{2
 Do the work for one invocation of hopen(1).  Dies on failure.  Main() then
 translates the die() into a print and error return.
 
+Takes a hash of options.
+
+The return value of _inner is unspecified and ignored.
+
 =cut
 
     my %opts = @_;
     local $_hrData = {};
 
+    # TODO FIXME.  This is a bit of a hack: Reset global variables on --fresh.
+    # Instead, App::hopen should be a class, and each instance should have its
+    # own data (I think).
+    if($opts{FRESH}) {
+        $_did_set_phase = false;
+    }
+
     if($opts{PRINT_VERSION}) {  # print version, raw and dotted
         if($App::hopen::VERSION =~ m<^([^\.]+)\.(\d{3})(\d{3})>) {
             printf "hopen version %d.%d.%d ($App::hopen::VERSION)\n", $1, $2, $3;
         } else {
             say "hopen $VERSION";
         }
-        say "App::hopen in: $INC{'App/hopen.pm'}" if $opts{VERBOSE} >= 1;
-        return EXIT_OK;
+        say "App::hopen in: $INC{'App/hopen.pm'}" if $VERBOSE >= 1;
+        return;
     }
 
+    # = Initialize filesystem-related build-system globals ==================
+
     # Start with the default phase unless one was specified.
     $Phase = $opts{PHASE} // $PHASES[0];
     die "Phase $Phase is not one of the ones I know about (" .
@@ -581,7 +727,7 @@ EOT
 
     # Prohibit builds if there's a MY.hopen.pl file in the project directory,
     # since those are the marker of a destination directory.
-    die <<EOT if -e $proj_dir->file(MYH);
+    if(-e $proj_dir->file(MYH)) { die <<EOT; }
 I'm sorry, but project directory ``$proj_dir'' appears to actually be a
 build directory --- it has a @{[MYH]} file.  If you really want to build
 here, remove or rename @{[MYH]} and run me again.
@@ -601,7 +747,7 @@ EOT
             if( $stat->mtime > $myhstat->mtime ||
                 $stat->ctime > $myhstat->ctime)
             {
-                say "Skipping out-of-date ``$myhopen''" unless $opts{QUIET};
+                say "Skipping out-of-date ``$myhopen''" unless $QUIET;
                 $myhopen = undef;
                 last;
             }
@@ -627,10 +773,6 @@ I can't find any hopen project files (.hopen.pl or *.hopen.pl) for
 project directory ``$proj_dir''.
 EOT
 
-    # = Initialize ==========================================================
-
-    say "From ``$proj_dir'' into ``$dest_dir''" unless $opts{QUIET};
-
     # Prepare the destination directory if it doesn't exist
     File::Path::Tiny::mk($dest_dir) or die "Couldn't create $dest_dir: $!";
 
@@ -638,39 +780,53 @@ EOT
     # generator and toolset can add initialization operations.
     $Build = hnew DAG => '__R_main';
 
+    # = Load generator and toolset (and run MYH) ============================
+
+    say "From ``$proj_dir'' into ``$dest_dir''" unless $QUIET;
+
     # Load MY.hopen.pl first so the results of the Probe phase are
     # available to the generator and toolset.
-    if($myhopen) {
+    if($myhopen && !$opts{BUILD}) {
         _execute_hopen_file($myhopen,
             forward_opts(\%opts, {lc=>1}, qw(PHASE QUIET)),
         );  # TODO support _e_h_f libs option
     }
 
     # Tell the user the initial phase if MY.hopen.pl didn't change it
-    say "Running $Phase phase" unless $_did_set_phase or $opts{QUIET};
+    say "Running $Phase phase" unless $opts{BUILD} or $_did_set_phase or $QUIET;
 
     # Load generator
-    my ($gen, $gen_class);
-    $gen_class = loadfrom($opts{GENERATOR}, 'App::hopen::Gen::', '');
-    die "Can't find generator $opts{GENERATOR}" unless $gen_class;
-    hlog { "Generator spec ``$opts{GENERATOR}'' -> using generator $gen_class" };
+    {
+        my ($gen, $gen_class);
+        $gen_class = loadfrom($opts{GENERATOR}, 'App::hopen::Gen::', '');
+        die "Can't find generator $opts{GENERATOR}" unless $gen_class;
+        hlog { "Generator spec ``$opts{GENERATOR}'' -> using generator $gen_class" };
 
-    $gen = "$gen_class"->new(proj_dir => $proj_dir, dest_dir => $dest_dir,
-        architecture => $opts{ARCHITECTURE})
-            or die "Can't initialize generator";
-    $Generator = $gen;
+        $gen = "$gen_class"->new(proj_dir => $proj_dir, dest_dir => $dest_dir,
+            architecture => $opts{ARCHITECTURE})
+                or die "Can't initialize generator";
+        $Generator = $gen;
+    }
 
     # Load toolset
-    my $toolset_class;
-    $opts{TOOLSET} //= $gen->default_toolset;
-    $toolset_class = loadfrom($opts{TOOLSET},
-                                    'App::hopen::T::', '');
-    die "Can't find toolset $opts{TOOLSET}" unless $toolset_class;
+    {
+        my $toolset_class;
+        $opts{TOOLSET} //= $Generator->default_toolset;
+        $toolset_class = loadfrom($opts{TOOLSET},
+                                        'App::hopen::T::', '');
+        die "Can't find toolset $opts{TOOLSET}" unless $toolset_class;
 
-    hlog { "Toolset spec ``$opts{TOOLSET}'' -> using toolset $toolset_class" };
-    $Toolset = $toolset_class;
+        hlog { "Toolset spec ``$opts{TOOLSET}'' -> using toolset $toolset_class" };
+        $Toolset = $toolset_class;
+    }
 
-    # = Run the hopen files =================================================
+    # Handle --build, now that everything's loaded --------------
+    if($opts{BUILD}) {
+        $Generator->run_build();
+        return;
+    }
+
+    # = Run the hopen files (except MYH, already run) =======================
 
     my $new_data;
     if(@$lrHopenFiles) {
@@ -695,7 +851,8 @@ EOT
 
     # TODO? give the generators a way to stash information that will be
     # written at the top of MY.hopen.pl.  This way, the user may only
-    # need to edit right at the top of the file, and not also at the
+    # need to edit right at the top of the file, and not also throughout
+    # the hashref.
 
     my $VAR = '__R_new_data';
     my $dumper = Data::Dumper->new([$new_data], [$VAR]);
@@ -757,7 +914,9 @@ Command-line runner.  Call as C<< App::hopen::Main(\@ARGV) >>.
 
     # Option overrides: -q beats -v
     $opts{VERBOSE} = 0 if $opts{QUIET};
-    $QUIET = $opts{QUIET} // false;
+    $QUIET = !!($opts{QUIET} // false);
+    delete $opts{QUIET};
+        # After this, code only refers to $QUIET for consistency.
 
     # Implement verbosity
     if(!$QUIET && $opts{VERBOSE}) {
@@ -765,11 +924,14 @@ Command-line runner.  Call as C<< App::hopen::Main(\@ARGV) >>.
         #hlog { Verbosity => $VERBOSE };
 
         # Under -v, keep stdout and stderr lines in order.
-        use IO::Handle;
-        STDOUT->autoflush;
-        STDERR->autoflush;
+        STDOUT->autoflush(true);
+        STDERR->autoflush(true);
     }
 
+    delete @opts{qw(VERBOSE VERBOSE2)};
+        # After this, code only refers to $QUIET for consistency.
+    # After th
+
     # Don't print the source of an eval'ed hopen file unless -vvv or higher.
     # Need 3 for the "..." that Carp prints when truncating.
     $Carp::MaxEvalLen = 3 unless $VERBOSE >= 3;
@@ -792,72 +954,7 @@ Command-line runner.  Call as C<< App::hopen::Main(\@ARGV) >>.
 
 1;
 __END__
-# === Command-line usage documentation ================================== {{{1
-
-=head1 OPTIONS
-
-=over
-
-=item -a C<architecture>
-
-Specify the architecture.  This is an arbitrary string interpreted by the
-generator or toolset.
-
-=item -e C<Perl code>
-
-Add the C<Perl code> as if it were a hopen file.  C<-e> files are processed
-after all other hopen files, so can modify anything that has been set up
-by those files.  Can be specified more than once.
-
-=item --fresh
-
-Start a fresh build --- ignore any C<MY.hopen.pl> file that may exist in
-the destination directory.
-
-=item --from C<project dir>
-
-Specify the project directory.  Overrides a project directory given as a
-positional argument.
-
-=item -g C<generator>
-
-Specify the generator.  The given C<generator> should be either a full package
-name or the part after C<App::hopen::Gen::>.
-
-=item -t C<toolset>
-
-Specify the toolset.  The given C<toolset> should be either a full package
-name or the part after C<App::hopen::T::>.
-
-=item --to C<destination dir>
-
-Specify the destination directory.  Overrides a destination directory given
-as a positional argument.
-
-=item --phase C<phase>
-
-Specify which phase of the process to run.  Note that this overrides whatever
-is specified in any MY.hopen.pl file, so may cause unexpected results!
-
-If C<--phase> is given, no other hopen file can set the phase, and hopen will
-terminate if a file attempts to do so.
-
-=item -q
-
-Produce no output (quiet).  Overrides C<-v>.
-
-=item -v, --verbose=n
-
-Verbose.  Specify more C<v>'s for more verbosity.  At present, C<-vv>
-(equivalently, C<--verbose=2>) gives
-you detailed traces of the data, and C<-vvv> gives you more detailed
-code tracebacks on error.
-
-=item --version
-
-Print the version of hopen and exit
-
-=back
+# === Rest of the documentation ========================================= {{{1
 
 =head1 AUTHOR
 
@@ -873,7 +970,7 @@ You can find documentation for this module with the perldoc command.
 
 You can also look for information at:
 
-=over 4
+=over
 
 =item * GitHub: The project's main repository and issue tracker
 
@@ -883,6 +980,10 @@ L<https://github.com/hopenbuild/App-hopen>
 
 L<https://metacpan.org/pod/App::hopen>
 
+=item * This distribution
+
+See the C<eg/> directory distributed with this software for examples.
+
 =back
 
 =head1 LICENSE AND COPYRIGHT
